from phone_iso3166.country import phone_country,country_prefix
import pycountry , phonenumbers
from threading import  Semaphore
from time import sleep
from colorama import Fore
import getpass
import os
from glob import glob
import shutil
from pathlib import Path

screenlock = Semaphore(value=1)

class SpaceXFile:
    def __init__(self):
        pass
    def create_folder(self, folder):
        """Creates a folder of the given name if it doesn't already exist."""
        if folder.endswith("/"):
            folder = folder[:-1]
        if len(folder) < 1:
            raise Exception("Minimum folder name length = 1.")
        if not os.path.exists(folder):
            try:
                os.makedirs(folder)
            except Exception:
                pass
                
    def create_file(self,filename):
        if Path(f'{filename}.txt').is_file():
            f = 0
        else:
            open(f'{filename}.txt' , 'w+')
                
    def clear_temp(self):
        usernamePath = getpass.getuser()
        Temp = f'C:/Users/{usernamePath}/AppData/Local/Temp/'
        lists = glob(f"{Temp}*")
        for l in lists:
            try:
                shutil.rmtree(l, ignore_errors=True)
            except:
                pass
            try:
                os.unlink(l)
            except:
                pass

    def SaveLineInFile(self,filename,line):
        sleep(0.1)
        if Path(f'{filename}.txt').is_file():
            f = 0
        else:
            open(f'{filename}.txt' , 'w+')
        print(line,file=open(f'{filename}.txt','a+'))

    def Usetextfile_Update(self,file_name,file_name_Update):
        screenlock.acquire()
        with open(f'{file_name}.txt','r+') as fin:
            lines = fin.readlines()
            fin.close()
        if len(lines) == 0 or lines[0] == '\n':
            with open(f'{file_name_Update}.txt','r') as fin:
                lines = fin.readlines()
                if len(lines) == 0 or lines[0] == '\n':
                    print(f'{Fore.RED}[!] Check The File {Fore.GREEN}"{file_name}.txt"{Fore.RED} Because It Is Empty ')
                    screenlock.release()
                    while 1 == 1:
                        sleep(99999)
                line_value = lines[0].strip()
                del lines[0]
                print(''.join(lines).strip(), file=open(f'{file_name}.txt', "w+"))
                fin.close()
            screenlock.release()
        else:
            line_value = lines[0].strip()
            del lines[0]
            print(''.join(lines).strip(), file=open(f'{file_name}.txt', "w+"))
            screenlock.release()
        return line_value
    def getNumber(self,file_name,file_name_Update):
        while True:
            phonenume  = self.Usetextfile_Update(file_name=file_name,file_name_Update=file_name_Update)
            readnumber = f"+{phonenume}"
            try:
                phone = phonenumbers.parse(readnumber).national_number
                region_code = phone_country(readnumber)
                prefix = country_prefix(region_code)
                name = pycountry.countries.get(alpha_2=region_code).name
                break
            except Exception as e:
                pass
        return phone , region_code , prefix , name , readnumber , phonenume


from time import sleep
from bs4 import BeautifulSoup

class SpaceXTempMail:
    def getEmailFromTempmail(self,session):
        while True:
            try:
                response = session.post('https://web2.temp-mail.org/mailbox')
                text = response.text       
                if not '","mailbox":"' in text:
                    jsonn = response.json()
                    errorMessage = jsonn['errorMessage']
                    print(errorMessage)
                    continue
                else:
                    jsonn = response.json()
                    tokenmailbox = jsonn['token']
                    mailbox = jsonn['mailbox']
                    break
            except Exception as e:
                print(e)
                pass
        return mailbox , tokenmailbox
            
    def getCodeFromTempmail(self,session,token,Keyword):
        headers = {'authorization': f'Bearer {token}'}    
        soup = None
        for i in range(30):
            sleep(1)
            req = session.get('https://web2.temp-mail.org/messages', headers=headers)
            if Keyword in req.text:
                json = req.json()
                messages = json['messages'][0]
                _id = messages['_id']
                response = session.get(f'https://web2.temp-mail.org/messages/{_id}', headers=headers)
                json = response.json()
                bodyHtml = json['bodyHtml']
                soup = BeautifulSoup(bodyHtml, 'html.parser')
                break
        return soup


try:
    import os,random
    from colorama import Fore
    from selenium.webdriver.common.by import By
    from selenium.webdriver import ActionChains
    from selenium.webdriver.support import expected_conditions as EC
    from selenium_profiles.webdriver import Chrome
    from selenium_profiles.profiles import profiles
    from selenium.webdriver.support.ui import WebDriverWait
    from selenium import webdriver
    from selenium.webdriver.chrome.service import Service
    import ua_generator
    
    import codecs
    import json
    from selenium.common.exceptions import (ElementClickInterceptedException as ECI_Exception,ElementNotInteractableException as ENI_Exception,InvalidArgumentException,MoveTargetOutOfBoundsException,NoSuchElementException,NoSuchWindowException,StaleElementReferenceException as Stale_Exception,TimeoutException,WebDriverException)
    from SpaceXRM.MyFileHandling import SpaceXFile
except Exception as e:
    print('------------------------------------------------------------------------------------------------')
    print(e)
    print('------------------------------------------------------------------------------------------------')
    input("Press ENTER to exit: ")
    quit()

class SpaceXSelenium(SpaceXFile):
    def __init__(self):
        self.driver     = None
        self.playwright = None
        #pass
        

    def ClickElement(self,selector,timeout):
        if 'text=' in selector:
            text = str(selector).split('="')[1].split('"')[0].strip()
            WebDriverWait(self.driver, timeout).until(EC.element_to_be_clickable((By.XPATH,f"//*[contains(text(), '{text}')]"))).click()
        else:
            WebDriverWait(self.driver, timeout).until(EC.element_to_be_clickable((By.CSS_SELECTOR,selector))).click()
        
    def SendKeysElement(self,selector,text,timeout):
        WebDriverWait(self.driver, timeout).until(EC.presence_of_element_located((By.CSS_SELECTOR,selector))).send_keys(str(text))
        
    def CreateActionChains(self,selector,timeout):
        element = WebDriverWait(self.driver, timeout).until(EC.presence_of_element_located((By.CSS_SELECTOR,selector)))
        actions = ActionChains(self.driver)
        return actions.move_to_element(element)


    def quit_all_driver(self):
        for driver in self.drivers_list:
            try:
                if (hasattr(driver, "service") and driver.service.process):
                    driver.quit()
            except AttributeError:
                pass
            except Exception:
                pass
    def save_page_source(self,name, folder=None):
        if not name.endswith(".html"):
            name = name + ".html"
        if folder:
            abs_path = os.path.abspath(".")
            file_path = os.path.join(abs_path, folder)
            if not os.path.exists(file_path):
                os.makedirs(file_path)
            html_file_path = os.path.join(file_path, name)
        else:
            html_file_path = name
        page_source = self.driver.page_source
        html_file = codecs.open(html_file_path, "w+", "utf-8")
        html_file.write(page_source)
        html_file.close()
    
    def save_cookies(self, name="cookies.txt"):
        """Saves the page cookies to the "saved_cookies" folder."""
        #self.wait_for_ready_state_complete()
        cookies = self.driver.get_cookies()
        json_cookies = json.dumps(cookies)
        if name.endswith("/"):
            raise Exception("Invalid filename for Cookies!")
        if "/" in name:
            name = name.split("/")[-1]
        if "\\" in name:
            name = name.split("\\")[-1]
        if len(name) < 1:
            raise Exception("Filename for Cookies is too short!")
        if not name.endswith(".txt"):
            name = name + ".txt"
        folder = 'Cookie'
        abs_path = os.path.abspath(".")
        file_path = os.path.join(abs_path, folder)
        if not os.path.exists(file_path):
            os.makedirs(file_path)
        cookies_file_path = os.path.join(file_path, name)
        cookies_file = codecs.open(cookies_file_path, "w+", encoding="utf-8")
        cookies_file.writelines(json_cookies)
        cookies_file.close()
    
    def load_cookies(self, name="cookies.txt"):
        """Loads the page cookies from the "saved_cookies" folder."""
        if name.endswith("/"):
            raise Exception("Invalid filename for Cookies!")
        if "/" in name:
            name = name.split("/")[-1]
        if "\\" in name:
            name = name.split("\\")[-1]
        if len(name) < 1:
            raise Exception("Filename for Cookies is too short!")
        if not name.endswith(".txt"):
            name = name + ".txt"
        folder = 'Cookie'
        abs_path = os.path.abspath(".")
        file_path = os.path.join(abs_path, folder)
        cookies_file_path = os.path.join(file_path, name)
        json_cookies = None
        with open(cookies_file_path, "r") as f:
            json_cookies = f.read().strip()
        cookies = json.loads(json_cookies)
        for cookie in cookies:
            if "expiry" in cookie:
                del cookie["expiry"]
            self.driver.add_cookie(cookie)
    
    def check_browser(self):
        """This method raises an exception if the active window is closed.
        (This provides a much cleaner exception message in this situation.)"""
        active_window = None
        try:
            active_window = self.driver.current_window_handle  # Fails if None
        except Exception:
            pass
        if not active_window:
            raise NoSuchWindowException("Active window was already closed!")
            
    def print_msg(self,namecolor,messege):
        print(f"{namecolor}{messege}")
    def GetChromeOptions(self,type_disply,Use_Proxy):     
        options = webdriver.ChromeOptions()
        options.add_argument("--incognito")
        options.add_argument("--lang=en-US")
        options.add_argument("--log-level=3")
        options.add_argument('--no-sandbox')
        options.add_argument("--window-size=400,800")
        options.add_argument('--disable-dev-shm-usage')
        #options.add_argument('--blink-settings=imagesEnabled=false')
        options.add_argument('--disable-blink-features=AutomationControlled') 
        #theuseragent = UserAgent(random.choice(['firefox','edge','ios'])).Random()
        theuseragent = ua_generator.generate(device='desktop', browser='firefox').text
        options.add_argument(f'--user-agent={theuseragent}')
        if type_disply == 'y' or type_disply == 'Y' or type_disply == 'غ':
            options.add_argument("--headless=new")
        if Use_Proxy == 'y' or Use_Proxy == 'Y' or Use_Proxy == 'غ':
            proxy  = self.Usetextfile_Update(file_name='Proxies',file_name_Update='ProxiesUpdate')
            proxy_type = 'http'
            options.add_argument(f'--proxy-server={proxy_type}://{proxy}')
            self.print_msg(Fore.GREEN,f'[-] Proxy Used  :: [{proxy}]')
        return options
        
    def GetDriverNormal(self,type_disply,Use_Proxy):
        options = self.GetChromeOptions(type_disply,Use_Proxy)
        options.add_experimental_option('useAutomationExtension', False)
        options.add_experimental_option("excludeSwitches", ["enable-automation","enable-logging"])
        self.driver = webdriver.Chrome(service=Service(executable_path='chromedriver.exe'),options=options)
        self.driver.execute_cdp_cmd("Page.addScriptToEvaluateOnNewDocument", {"source": """Object.defineProperty(navigator, 'webdriver', {get: () => undefined})""",})
        self.driver.set_page_load_timeout(100)
        return self.driver
    
    def GetDriverFromProfiles(self,Typeprofiles,uc_driver,type_disply,Use_Proxy):
        if Typeprofiles == 1:
            profile = profiles.Android()
        else:
            profile = profiles.Windows()
        options = self.GetChromeOptions(type_disply,Use_Proxy)
        if uc_driver == False:
            options.add_experimental_option("excludeSwitches", ["enable-automation", "enable-logging"])
        try:
            self.driver  = Chrome(profile,options=options,executable_path='chromedriver.exe',uc_driver=uc_driver)
        except:
            self.driver  = Chrome(profile,options=options,executable_path='chromedriver.exe',chrome_binary='chromedriver.exe',uc_driver=uc_driver)
        if uc_driver == True:
            self.driver.set_window_position(random.randint(111,999),random.randint(000,600))
        
        self.driver.execute_cdp_cmd("Page.addScriptToEvaluateOnNewDocument", {"source": """Object.defineProperty(navigator, 'webdriver', {get: () => undefined})""",})
        self.driver.set_page_load_timeout(100)
        return self.driver
    
    def GetDriverFromBlaywright(self,headless_disply,Use_Proxy):
        proxy = None
        currentproxy = None
        if Use_Proxy == 'y' or Use_Proxy == 'Y' or Use_Proxy == 'غ':
            prox = self.Usetextfile_Update(file_name='Proxies',file_name_Update='ProxiesUpdate')
            currentproxy = {'http': f'http://{prox}','https': f'http://{prox}'}
            proxy = {"server": f"http://{prox}"}
            self.print_color(Fore.GREEN,f'[-] Proxy Used  :: [{proxy}]')
        #playwright = sync_playwright().start()
        self.browser = self.playwright.chromium.launch(headless=headless_disply,proxy=proxy)
        useragent = ua_generator.generate(device='mobile').text
        self.driver = self.browser.new_page(viewport={"width":600,"height":800},locale='en-US',user_agent=useragent)
        return self.driver


from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))
with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = '0.0.4'
DESCRIPTION = 'Special package'
# LONG_DESCRIPTION = 'A package that allows to build simple streams of video, audio and camera data.'

# Setting up
setup(
    name="SpaceXRM",
    version=VERSION,
    author="Ramiz Al-Jabri",
    author_email="<ramzalgabry232@gmail.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(),
    install_requires=['pycountry', 'phonenumbers', 'phone_iso3166', 'colorama', 'curl_cffi', 'selenium_profiles', 'playwright', 'selenium', 'ua_generator', 'bs4','names'],
    keywords=['python', 'selenium', 'playwright', 'selenium_profiles', 'curl_cffi', 'requests'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Microsoft :: Windows"
    ]
)



try:
    import os , requests
    import curl_cffi.requests as requests2 
    from colorama import Fore
    from SpaceXRM.MyFileHandling import SpaceXFile
    from SpaceXRM.MyTempMail import SpaceXTempMail
except Exception as e:
    print('------------------------------------------------------------------------------------------------')
    print(e)
    print('------------------------------------------------------------------------------------------------')
    input("Press ENTER to exit: ")
    quit()

class SpaceXRequests(SpaceXFile,SpaceXTempMail):
    def __init__(self):
        self.session = None
        #pass

    def check_connection(self) -> None:
        try:
            self.session.get("https://google.com")
        except requests.RequestException:
            self.print_msg(Fore.RED,"Error while connecting to the Internet. Make sure you are connected to the Internet!")

    def save_page_source(self,name,folder,page_source):
        if not name.endswith(".html"):
            name = name + ".html"
        if folder:
            abs_path = os.path.abspath(".")
            file_path = os.path.join(abs_path, folder)
            if not os.path.exists(file_path):
                os.makedirs(file_path)
            html_file_path = os.path.join(file_path, name)
        else:
            html_file_path = name
        with open(html_file_path, "wb") as file:
            file.write(page_source) 
    def print_msg(self,namecolor,messege):
        print(f"{namecolor}{messege}")

    def GetRequestsSessionNormal(self,Use_Proxy,socks,currentproxy):
        self.session = requests.Session()
        if Use_Proxy == 'y' or Use_Proxy == 'Y' or Use_Proxy == 'غ':
            if currentproxy == None:
                currentproxy = self.Usetextfile_Update(file_name='Proxies',file_name_Update='ProxiesUpdate')
            try:
                userp , passw , ip , port = currentproxy.split(':')
                proxy = f'{userp}:{passw}@{ip}:{port}'
            except:
                ip , port = currentproxy.split(':')
                proxy = f'{ip}:{port}'
            if socks == True:
                self.session.proxies.update({'http': f'socks5://{proxy}', 'https': f'socks5://{proxy}'})
            else:
                self.session.proxies.update({'http': f'http://{proxy}', 'https': f'http://{proxy}'})
        return self.session

    def GetRequestsSessionCurl_cffi(self,Use_Proxy,socks,currentproxy):
        self.session = requests2.Session(impersonate="chrome120",timeout=5)
        if Use_Proxy == 'y' or Use_Proxy == 'Y' or Use_Proxy == 'غ':
            if currentproxy == None:
                currentproxy = self.Usetextfile_Update(file_name='Proxies',file_name_Update='ProxiesUpdate')
            try:
                userp , passw , ip , port = currentproxy.split(':')
                currentproxy = f'{userp}:{passw}@{ip}:{port}'
            except:
                ip , port = currentproxy.split(':')
                currentproxy = f'{ip}:{port}'
            if socks == True:
                self.session.proxies.update({'http': f'socks5://{currentproxy}', 'https': f'socks5://{currentproxy}'})
            else:
                self.session.proxies.update({'http': f'http://{currentproxy}', 'https': f'http://{currentproxy}'})
        return self.session